//==========================================================================
#include <owl.h>      // Simple example program that shows how to hook the
#include <mmsystem.h> // Keyboard.  Uses hoodll.dll for the hook function
		      // that calls back to the window in this .exe
		// released to the public domain 10/30/92 by Bob Bourbonnais
LRESULT CALLBACK _export MyKbFilter(int nCode,       // Prototype for hook
				    WPARAM wParam,   // function in the DLL
				    LPARAM lParam);
//==========================================================================
_CLASSDEF(TMyWindow)                                 // My Window Class
class TMyWindow : public TWindow
{
public:
  TMyWindow(PTWindowsObject AParent, LPSTR ATitle);  // Constructor
  ~TMyWindow();                                      // Destructor
  virtual void GetWindowClass(WNDCLASS&);            // Changes class
  void WMSound(RTMessage Msg)           = [WM_USER]; // User DDVT called by
private:                                             // DLL
  BOOL fKeyDown;                                     // Keydown flag
  HHOOK kbHook;                                      // Handle to the Hook
};
//--------------------------------------------------------------------------
TMyWindow::TMyWindow(PTWindowsObject AParent, // Constructor calls the
		     LPSTR ATitle)
	  :TWindow(AParent, ATitle)           // Base class constructor
  {
  fKeyDown     = FALSE;                       // key is up
  Attr.X       = 100;                         // window coordinates
  Attr.Y       = 100;
  Attr.W       = 200;
  Attr.H       = 50;
  mciSendString((LPSTR)"open OwlHoo.wav alias OwlHoo", // Load wave file
		 NULL,80,NULL);
  kbHook = SetWindowsHookEx(WH_KEYBOARD,MyKbFilter,    // activate keyboard
	   GetApplication()->hInstance,NULL);          // hook function
  };
//--------------------------------------------------------------------------
TMyWindow::~TMyWindow()                                // Destructor
  {
  mciSendString((LPSTR)"close OwlHoo",NULL,80,NULL);   // unload wave file
  UnhookWindowsHookEx(kbHook);                         // stop hooking
  }                                                    // keyboard
//--------------------------------------------------------------------------
void TMyWindow::GetWindowClass( WNDCLASS& AWndClass )  // change class
  {                                                    // to use special
  TWindow::GetWindowClass( AWndClass );                // icon
  AWndClass.hIcon = LoadIcon( GetApplication()->hInstance, "OwlHooIcon" );
  }
//--------------------------------------------------------------------------
void TMyWindow::WMSound(RTMessage Msg)  // The hook function in the DLL
  {                         // calls this function by posting a message
  const long KEYUP = 0x80000000L;       // top bit signifies the key up
  if ((!fKeyDown) && !(Msg.LParam & KEYUP)) // keypressed
    {
    mciSendString ((LPSTR)"play OwlHoo from 1", // Play a sound
		    NULL, 80, NULL);
    char myBuff[40];                            // Output the key that was
    HDC hdc = GetDC(HWindow);                   // Pressed
    wsprintf(myBuff,"The Key %c has a value = %i   ",Msg.WParam,Msg.WParam);
    TextOut(hdc,0,0,myBuff,lstrlen(myBuff));
    ReleaseDC(HWindow,hdc);
    fKeyDown = TRUE;
    }
  else if ((fKeyDown) && (Msg.LParam && KEYUP))  // Key is up then stop the
    {                                            // Sound
    mciSendString ((LPSTR)"stop OwlHoo",NULL,0,NULL);
    fKeyDown = FALSE;
    }
  }
//==========================================================================
class TMyApp : public TApplication          // application to override
{                                           // InitMainWindow
public:
  TMyApp(LPSTR AName, HINSTANCE hInstance,  // Constructor calls base
	 HINSTANCE hPrevInstance,           // class
	 LPSTR lpCmdLine, int nCmdShow)
	 : TApplication(AName,hInstance,hPrevInstance,lpCmdLine,nCmdShow)
	 {};
  virtual void InitMainWindow();
};
//--------------------------------------------------------------------------
void TMyApp::InitMainWindow()
  {
  MainWindow = new TMyWindow(NULL, Name); // Create an instance of My Window
  }
//==========================================================================
int PASCAL WinMain(HINSTANCE hInstance,   // Program entry point
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
  {
  TMyApp MyApp("OwlHoo", hInstance, hPrevInstance, // create instance of
		 lpCmdLine,nCmdShow);              // My Application
  MyApp.Run();                                     // Run it
  return MyApp.Status;                             // Finished
  }
//==========================================================================
